package com.android.servicetime;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;

import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.IBinder;
import android.util.Log;

public class LocationService extends Service implements LocationListener
{
	private Location prevLocation;
	private LocationManager lm;
	private DatagramSocket socket;
	
	private class Packet
	{
		double latitude;
		double longitude;
		double time;
		int query;
		int report;
		int result;
	}
	
	@Override
	public IBinder onBind(Intent intent)
	{
		return null;
	}

	@Override
	public void onCreate()
	{
		super.onCreate();
		lm = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
		double res = Double.parseDouble(getResources().getString(R.string.gps_resolution));
		long upd_time = Long.parseLong(getResources().getString(R.string.gps_update_time));
		lm.requestLocationUpdates(LocationManager.GPS_PROVIDER, upd_time, (float)res,
				this);
		try
		{
			socket = new DatagramSocket();
			socket.setSoTimeout(1000);
		} catch (SocketException se)
		{
		}
	}
	
	public void onDestroy()
	{
		super.onDestroy();
		socket.close();
		lm.removeUpdates(this);
		Log.d("debug", "Service Stopped!!");
	}

	public void onLocationChanged(Location location)
	{
		long timespent = 0;
		
		if (prevLocation == null)
		{
			prevLocation = location;
		}
		else
		{
			timespent = (location.getTime() - prevLocation.getTime())/1000;
			try
			{
				Packet loc_pkt = new Packet();
				loc_pkt.latitude = prevLocation.getLatitude();
				loc_pkt.longitude = prevLocation.getLongitude();
				loc_pkt.time = timespent;
				loc_pkt.report = 1; //Reporting Change of location
				loc_pkt.query = 0;
				loc_pkt.result = 0;
				
				ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
				DataOutputStream dataStream = new DataOutputStream(byteStream);
				dataStream.writeDouble(loc_pkt.latitude);
				dataStream.writeDouble(loc_pkt.longitude);
				dataStream.writeDouble(loc_pkt.time);
				dataStream.writeInt(loc_pkt.query);
				dataStream.writeInt(loc_pkt.report);
				dataStream.writeInt(loc_pkt.result);
				dataStream.flush();
				byte[] bytes = byteStream.toByteArray();
				
				String srvaddress = getResources().getString(R.string.serv_address);
				DatagramPacket pkt = new DatagramPacket(bytes, bytes.length, InetAddress.getByName(srvaddress), 9930);
				socket.send(pkt);
			}
			catch(SocketException se) {}
			catch(IOException ioe) {}
		}
	}

	public void onProviderDisabled(String provider)
	{
	}

	public void onProviderEnabled(String provider)
	{
	}

	public void onStatusChanged(String provider, int status, Bundle extras)
	{
	}
}
