package com.android.servicetime;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;

import android.app.Activity;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Point;
import android.view.MotionEvent;
import android.widget.Toast;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapView;

public class ServiceOverlay extends com.google.android.maps.Overlay
{
	GeoPoint gp;
	Activity mapactivity;
	DatagramSocket socket;

	public ServiceOverlay(Activity act, GeoPoint p)
	{
		this.gp = p;
		this.mapactivity = act;
	}

	@Override
	public boolean draw(Canvas canvas, MapView mapView, boolean shadow,
			long when)
	{
		super.draw(canvas, mapView, shadow);

		// ---translate the GeoPoint to screen pixels---
		Point screenPts = new Point();
		mapView.getProjection().toPixels(gp, screenPts);

		// ---add the marker---
		Bitmap bmp = BitmapFactory.decodeResource(mapactivity.getResources(),
				R.drawable.fing4_n);
		//canvas.drawBitmap(bmp, screenPts.x, screenPts.y - 50, null);
		return true;
	}

	@Override
	public boolean onTouchEvent(MotionEvent event, MapView mapView)
	{
		int servtime = 0;
		String str;
		
		if (event.getAction() == 1)
		{
			GeoPoint p = mapView.getProjection().fromPixels((int) event.getX(),
					(int) event.getY());
			sendSelectedLocation(p); // Send Query to server
			servtime = (int) getServiceTime();
			if (servtime == 0)
			{
				str = new String("No Service Time for this location");
			}
			else if (servtime > 60)
			{
				str = Integer.toString((int)(servtime/60)) + " min";
			}
			else
			{
				str = Integer.toString(servtime) + " secs";
			}
			
			Toast.makeText(mapactivity.getBaseContext(), str,
					Toast.LENGTH_SHORT).show(); // Display result
		}
		return false;
	}
	
	private void sendSelectedLocation(GeoPoint p)
	{
		try
		{
			socket = new DatagramSocket();
			socket.setSoTimeout(1000);
			ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
			DataOutputStream dataStream = new DataOutputStream(byteStream);
			dataStream.writeDouble(p.getLatitudeE6()/1E6);
			dataStream.writeDouble(p.getLongitudeE6()/1E6);
			dataStream.writeDouble(0);
			dataStream.writeInt(1); //Query
			dataStream.writeInt(0); //Report
			dataStream.writeInt(0); //Result
			dataStream.flush();
			byte[] bytes = byteStream.toByteArray();
			
			String srvaddress = mapactivity.getBaseContext().getResources().getString(R.string.serv_address);

			DatagramPacket pkt = new DatagramPacket(bytes, bytes.length,
					InetAddress.getByName(srvaddress), 9930);
			socket.send(pkt);
		} catch (IOException ioe)
		{
		}
	}
	
	private double getServiceTime()
	{
		double timeSpent = 0;
		
		try
		{
			byte[] buf = new byte[256];
			DatagramPacket packet = new DatagramPacket(buf, buf.length);
			socket.receive(packet);
			socket.close();
			ByteArrayInputStream bis = new ByteArrayInputStream(buf,0,36);
			DataInputStream dis = new DataInputStream(bis);
			dis.readDouble();
			dis.readDouble();
			timeSpent = dis.readDouble();
		}
		catch(IOException ioe) {}
		return timeSpent;
	}
}
