#include <arpa/inet.h>
#include <netinet/in.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <string.h>

#define BUFLEN 512
#define NPACK 10
#define PORT 9930

struct packet
{
	double x;
	double y;
	double seconds;
	int query;
	int report;
	int result;
};
typedef struct packet packet;

struct record
{
	double x;
	double y;
	double seconds;
	unsigned int count;
	struct record *link;
};
typedef struct record record;

struct poi
{
	double x;
	double y;
};

struct record *head;
float alpha;
double resolution;
unsigned int valid_rec;

void diep(char *s);
double unswap_double(double a);
int unswap_int(int a);
struct packet conv_host_order(struct packet pkt);
int add_record(double x,double y, double seconds);
void print_pkt(struct packet pkt);
void loaddb();

void initdb()
{
	head=NULL;
	alpha=0.9;
	resolution=0.00015; //Translates to about 10 metres
	//resolution = 0.001;
	valid_rec=1;
}

void loaddb()
{
        FILE *fp;
         record rec;
        fp=fopen("./db", "r");
        if(fp==NULL)
        {
                printf("Could not open file\r\n");
                return;
        }

        while(fread(&rec,1,sizeof(record),fp)==sizeof(record))
        {
		add_record(rec.x,rec.y,rec.seconds);
        }
        fclose(fp);

}

record *search_record(double x, double y)
{
	record *tmp;
	tmp=head;
	while(tmp)
	{
		if((tmp->x==x) && (tmp->y==y))
		{
			return tmp;	
		}
		tmp=tmp->link;	
	}
	
	return NULL;
}

double rec_query(double x,double y)
{
	record *tmp=head;
	double value=0.0;

	printf("Searching\n");
	while(tmp)
	{
		printf("Entry :: %.14f, %.14f, %f\n", tmp->x,tmp->y,tmp->seconds);
		if(tmp->x<(x+resolution)&&tmp->x>(x-resolution))
		{
			if(tmp->y<(y+resolution)&&tmp->y>(y-resolution))
			{
				printf("Found Entry\n");
				if(tmp->count>=valid_rec)
				{
					value = tmp->seconds;
					break;
				}
			}
		}
		tmp=tmp->link;
	}
		
	return value;
}

// unswap using char pointers
double unswap_double(double a)
{
    double d;
    unsigned char *src = (unsigned char *)&a;
    unsigned char *dst = (unsigned char *)&d;

    dst[0] = src[7];
    dst[1] = src[6];
    dst[2] = src[5];
    dst[3] = src[4];
    dst[4] = src[3];
    dst[5] = src[2];
    dst[6] = src[1];
    dst[7] = src[0];

    return d;
}

int unswap_int(int a)
{
	int i;
	unsigned char *src = (unsigned char *)&a;
	unsigned char *dst = (unsigned char *)&i;

	dst[0] = src[3];
	dst[1] = src[2];
	dst[2] = src[1];



	dst[3] = src[0];

	return i;
}

struct packet conv_host_order(struct packet pkt)
{
	struct packet conv_pkt;

	conv_pkt.x = unswap_double(pkt.x);
	conv_pkt.y = unswap_double(pkt.y);
	conv_pkt.seconds = unswap_double(pkt.seconds);
	conv_pkt.query = unswap_int(pkt.query);
	conv_pkt.report = unswap_int(pkt.report);
	conv_pkt.result = unswap_int(pkt.result);
	return conv_pkt;
}

int process_packet(struct sockaddr_in si,struct packet pkt )
{
			
	double seconds;
	int s;

	if(pkt.query==1)
	{
		printf("Query Received!!\n");
		print_pkt(pkt);
		seconds=rec_query(pkt.x,pkt.y);
		pkt.seconds = unswap_double(seconds);
		pkt.result=1;
		
		printf("Service time:: %lu\n", (unsigned long)pkt.seconds);

		if ((s=socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP))==-1)
		{
			diep("socket");
			return -1;
		}


		if (sendto(s, &pkt, sizeof(packet), 0, (struct sockaddr *)&si,sizeof(struct sockaddr_in) )==-1)
		{
			diep("sendto()");
			return -1;
		}



	}
	else if(pkt.report==1)
	{
		
		add_record(pkt.x,pkt.y,pkt.seconds);
	}
	else
	{
		printf("Invalid packet received\n");
	}
	return 0;
}


int add_record(double x,double y, double seconds)
{
	record *tmp=NULL;

	tmp=search_record(x,y);
	if(tmp!=NULL)
	{
		tmp->seconds=(alpha*(tmp->seconds))+((1.0-alpha)*seconds);
		tmp->count++;
		return 0;
	}
	
	tmp=malloc(sizeof(record));
		
	tmp->x=x;
	tmp->y=y;
	tmp->seconds=seconds;
	tmp->count=1;
	tmp->link=head;
	head=tmp;

	return 0;
}

void diep(char *s)
{
perror(s);
exit(1);
}

void print_pkt(struct packet pkt)
{
	printf("X: %.14f\n", pkt.x);
	printf("Y: %.14f\n", pkt.y);
	printf("Seconds: %u\n", (unsigned int)pkt.seconds);
	printf("Query: %d\n", pkt.query);
	printf("Report: %d\n", pkt.report);
	printf("Result: %d\n", pkt.result);
}

int main(void)
{
struct sockaddr_in si_me, si_other;
int s, rcvd;
socklen_t slen=sizeof(si_other);
//char buf[BUFLEN];
struct packet pkt, conv_pkt;
initdb();
loaddb();

if ((s=socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP))==-1)
diep("socket");

memset((char *) &si_me, 0, sizeof(si_me));
si_me.sin_family = AF_INET;
si_me.sin_port = htons(PORT);
si_me.sin_addr.s_addr = htonl(INADDR_ANY);
if (bind(s, (struct sockaddr *)&si_me, sizeof(si_me))==-1)
diep("bind");

while(1) {
if ((rcvd = recvfrom(s, &pkt,sizeof(packet), 0, (struct sockaddr *)&si_other, &slen))==-1)
diep("recvfrom()");
conv_pkt = conv_host_order(pkt);
printf("Received packet from %s:%d\n\n", 
inet_ntoa(si_other.sin_addr), ntohs(si_other.sin_port));
process_packet(si_other,conv_pkt);
}

close(s);
return 0;
}

